#pragma once
#include "FBullCowGame.h"
#include <map>

// to make syntax Unreal friendly
#define TMap std::map
using int32 = int;

void FBullCowGame::Reset()
{
	int32 num = rand() % 8 + 1;

	if (num == 1) {
		const FString HIDDEN_WORD = "ant"; // must be an isogram
		MyHiddenWord = HIDDEN_WORD;
	}
	else if (num == 2) {
		const FString HIDDEN_WORD = "pony"; // must be an isogram
		MyHiddenWord = HIDDEN_WORD;
	}
	else if (num == 3){
		const FString HIDDEN_WORD = "venom"; // must be an isogram
		MyHiddenWord = HIDDEN_WORD;
	}
	else if (num == 4) {
		const FString HIDDEN_WORD = "rounds"; // must be an isogram
		MyHiddenWord = HIDDEN_WORD;
	}
	else if (num == 5){
		const FString HIDDEN_WORD = "bengal"; // must be an isogram
		MyHiddenWord = HIDDEN_WORD;
	}
	else if (num == 6) {
		const FString HIDDEN_WORD = "art"; // must be an isogram
		MyHiddenWord = HIDDEN_WORD;
	}
	else if (num == 7) {
		const FString HIDDEN_WORD = "fruit"; // must be an isogram
		MyHiddenWord = HIDDEN_WORD;
	}
	else if (num == 8) {
		const FString HIDDEN_WORD = "star"; // must be an isogram
		MyHiddenWord = HIDDEN_WORD;
	}
	else {
		const FString HIDDEN_WORD = "planet"; // must be an isogram
		MyHiddenWord = HIDDEN_WORD;
	}

	
	MyCurrentTry = 1;
	bGameIsWon = false;
	return;
}

//receives a VALID guess, increments turn, and returns count
FBullCowCount FBullCowGame::SubmitValidGuess(FString Guess)
{
	MyCurrentTry++;
	FBullCowCount BullCowCount;
	int32 WordLength = MyHiddenWord.length(); //assuming same length as guess

	//loop through all letters in the hidden word
	for (int32 MHWChar = 0; MHWChar < WordLength; MHWChar++) {
	
		// compare letters against guess
		for (int32 GChar = 0; GChar < WordLength; GChar++) {
			//if they match then
			if (Guess[GChar] == MyHiddenWord[MHWChar]) {
				
				if (MHWChar == GChar) {// if they're in the same place
					BullCowCount.Bulls++;//incriment bulls
				}
				else {
					BullCowCount.Cows++; //incriment cows if not
				}
				
					
			}
		}
	}
	if (BullCowCount.Bulls == WordLength){
		bGameIsWon = true;
	}
	else {
		bGameIsWon = false;
	}
	return BullCowCount;
}

bool FBullCowGame::IsIsogram(FString Word) const
{
	//Treat 0 and 1 letter words as isograms
	if (Word.length() <= 1) {
		return true;
	}

	TMap<char, bool> LetterSeen;// setup our map
	for (auto Letter : Word) // for all letters of the word
	{
		Letter = tolower(Letter); // handle mixed case
		
		if (LetterSeen[Letter])//if the letter is in the map
		{
			return false; // we do NOT have an isogram
		}
		else {
			LetterSeen[Letter] = true; //add the letter to the map as seen
		}
				
	}

	return true; // for example in cases where /0 is entered
}

bool FBullCowGame::IsLowercase(FString Word) const
{
	for (auto Letter : Word)
	{
		if (!islower(Letter)) {// if not a lowercase letter
			return false;
		}
	}
	return true;
}

FBullCowGame::FBullCowGame() // The default constructor
{
	Reset();
}

int32 FBullCowGame::GetMaxTries() const
{
	TMap<int32, int32> WordLengthToMaxTries{ {3, 3}, {4, 5}, {5,7}, {6,8} };
	return WordLengthToMaxTries[MyHiddenWord.length()];
}

int32 FBullCowGame::GetCurrentTry() const
{
	return MyCurrentTry;
}

int32 FBullCowGame::GetHiddenWordLength() const
{
	return MyHiddenWord.length();
}

bool FBullCowGame::IsGameWon() const
{
	return bGameIsWon;
}

EGuessStatus FBullCowGame::CheckGuessValidity(FString Guess) const
{
	if (!IsIsogram(Guess))//If guess isn't an isogram
	{
		return EGuessStatus::Not_Isogram;
	}
	else if (!IsLowercase(Guess)) //if the guess isn't all lowercase
	{
		return EGuessStatus::Not_Lowercase;
	}
	else if (Guess.length() != GetHiddenWordLength())//if the guess lenght is wrong
	{
		return EGuessStatus::Wrong_Length;
	}
	else //otherwise
	{
		return EGuessStatus::OK;
	}

}
